"""Test QtPrintSupport."""

import sys

import pytest

from qtpy import QtPrintSupport


def test_qtprintsupport():
    """Test the qtpy.QtPrintSupport namespace"""
    assert QtPrintSupport.QAbstractPrintDialog is not None
    assert QtPrintSupport.QPageSetupDialog is not None
    assert QtPrintSupport.QPrintDialog is not None
    assert QtPrintSupport.QPrintPreviewDialog is not None
    assert QtPrintSupport.QPrintEngine is not None
    assert QtPrintSupport.QPrinter is not None
    assert QtPrintSupport.QPrinterInfo is not None
    assert QtPrintSupport.QPrintPreviewWidget is not None


def test_qpagesetupdialog_exec_():
    """Test qtpy.QtPrintSupport.QPageSetupDialog exec_"""
    assert QtPrintSupport.QPageSetupDialog.exec_ is not None


def test_qprintdialog_exec_():
    """Test qtpy.QtPrintSupport.QPrintDialog exec_"""
    assert QtPrintSupport.QPrintDialog.exec_ is not None


def test_qprintpreviewwidget_print_(qtbot):
    """Test qtpy.QtPrintSupport.QPrintPreviewWidget print_"""
    assert QtPrintSupport.QPrintPreviewWidget.print_ is not None
    preview_widget = QtPrintSupport.QPrintPreviewWidget()
    preview_widget.print_()



# -----------------------------------------------------------------------------
# Copyright © 2009- The Spyder Development Team
#
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
# -----------------------------------------------------------------------------

"""Provides QtXml classes and functions."""

from . import PYQT5, PYQT6, PYSIDE2, PYSIDE6

if PYQT5:
    from PyQt5.QtXml import *
elif PYQT6:
    from PyQt6.QtXml import *
elif PYSIDE2:
    from PySide2.QtXml import *
elif PYSIDE6:
    from PySide6.QtXml import *



import pytest
from packaging import version

from qtpy import PYQT5, PYQT6, PYQT_VERSION, PYSIDE2, PYSIDE6, PYSIDE_VERSION
from qtpy.tests.utils import pytest_importorskip


@pytest.mark.skipif(
    not (
        (PYQT6 and version.parse(PYQT_VERSION) >= version.parse("6.2"))
        or (PYSIDE6 and version.parse(PYSIDE_VERSION) >= version.parse("6.2"))
        or PYQT5
        or PYSIDE2
    ),
    reason="Only available in Qt<6,>=6.2 bindings",
)
def test_qtwebenginewidgets():
    """Test the qtpy.QtWebEngineWidget namespace"""

    QtWebEngineWidgets = pytest_importorskip("qtpy.QtWebEngineWidgets")

    assert QtWebEngineWidgets.QWebEnginePage is not None
    assert QtWebEngineWidgets.QWebEngineView is not None
    assert QtWebEngineWidgets.QWebEngineSettings is not None
    assert QtWebEngineWidgets.QWebEngineScript is not None



# -----------------------------------------------------------------------------
# Copyright © 2023- The Spyder Development Team
#
# Released under the terms of the MIT License
# (see LICENSE.txt for details)
# -----------------------------------------------------------------------------

"""Provides utility functions for use by QtPy itself."""
from functools import wraps
from typing import TYPE_CHECKING

import qtpy

if TYPE_CHECKING:
    from qtpy.QtWidgets import QAction


def _wrap_missing_optional_dep_error(
    attr_error,
    *,
    import_error,
    wrapper=qtpy.QtModuleNotInstalledError,
    **wrapper_kwargs,
):
    """Create a __cause__-chained wrapper error for a missing optional dep."""
    qtpy_error = wrapper(**wrapper_kwargs)
    import_error.__cause__ = attr_error
    qtpy_error.__cause__ = import_error
    return qtpy_error


def getattr_missing_optional_dep(name, module_name, optional_names):
    """Wrap AttributeError in a special error if it matches."""
    attr_error = AttributeError(
        f"module {module_name!r} has no attribute {name!r}",
    )
    if name in optional_names:
        return _wrap_missing_optional_dep_error(
            attr_error,
            **optional_names[name],
        )
    return attr_error


def possibly_static_exec(cls, *args, **kwargs):
    """Call `self.exec` when `self` is given or a static method otherwise."""
    if not args and not kwargs:
        # A special case (`cls.exec_()`) to avoid the function resolving error
        return cls.exec()
    if isinstance(args[0], cls):
        if len(args) == 1 and not kwargs:
            # A special case (`self.exec_()`) to avoid the function resolving error
            return args[0].exec()
        return args[0].exec(*args[1:], **kwargs)

    return cls.exec(*args, **kwargs)


def possibly_static_exec_(cls, *args, **kwargs):
    """Call `self.exec` when `self` is given or a static method otherwise."""
    if not args and not kwargs:
        # A special case (`cls.exec()`) to avoid the function resolving error
        return cls.exec_()
    if isinstance(args[0], cls):
        if len(args) == 1 and not kwargs:
            # A special case (`self.exec()`) to avoid the function resolving error
            return args[0].exec_()
        return args[0].exec_(*args[1:], **kwargs)

    return cls.exec_(*args, **kwargs)


def set_shortcut(self, shortcut, old_set_shortcut):
    """Ensure that the type of `shortcut` is compatible to `QAction.setShortcut`."""
    from qtpy.QtCore import Qt
    from qtpy.QtGui import QKeySequence

    if isinstance(shortcut, (QKeySequence.StandardKey, Qt.Key, int)):
        shortcut = QKeySequence(shortcut)
    old_set_shortcut(self, shortcut)


def set_shortcuts(self, shortcuts, old_set_shortcuts):
    """Ensure that the type of `shortcuts` is compatible to `QAction.setShortcuts`."""
    from qtpy.QtCore import Qt
    from qtpy.QtGui import QKeySequence

    if isinstance(
        shortcuts,
        (QKeySequence, QKeySequence.StandardKey, Qt.Key, int, str),
    ):
        shortcuts = (shortcuts,)

    shortcuts = tuple(
        (
            QKeySequence(shortcut)
            if isinstance(shortcut, (QKeySequence.StandardKey, Qt.Key, int))
            else shortcut
        )
        for shortcut in shortcuts
    )
    old_set_shortcuts(self, shortcuts)


def add_action(self, *args, old_add_action):
    """Re-order arguments of `addAction` to backport compatibility with Qt>=6.3."""
    from qtpy.QtCore import QObject, Qt
    from qtpy.QtGui import QIcon, QKeySequence

    action: QAction
    icon: QIcon
    text: str
    shortcut: QKeySequence | QKeySequence.StandardKey | Qt.Key | str | int
    receiver: QObject
    member: bytes

    if all(
        isinstance(arg, t)
        for arg, t in zip(
            args,
            [
                str,
                (QKeySequence, QKeySequence.StandardKey, Qt.Key, str, int),
                QObject,
                bytes,
            ],
        )
    ):
        if len(args) == 2:
            text, shortcut = args
            action = old_add_action(self, text)
            action.setShortcut(shortcut)
        elif len(args) == 3:
            text, shortcut, receiver = args
            action = old_add_action(self, text, receiver)
            action.setShortcut(shortcut)
        elif len(args) == 4:
            text, shortcut, receiver, member = args
            action = old_add_action(self, text, receiver, member, shortcut)
        else:
            action = old_add_action(self, *args)
    elif all(
        isinstance(arg, t)
        for arg, t in zip(
            args,
            [
                QIcon,
                str,
                (QKeySequence, QKeySequence.StandardKey, Qt.Key, str, int),
                QObject,
                bytes,
            ],
        )
    ):
        if len(args) == 3:
            icon, text, shortcut = args
            action = old_add_action(self, icon, text)
            action.setShortcut(shortcut)
        elif len(args) == 4:
            icon, text, shortcut, receiver = args
            action = old_add_action(self, icon, text, receiver)
            action.setShortcut(shortcut)
        elif len(args) == 5:
            icon, text, shortcut, receiver, member = args
            action = old_add_action(
                self,
                icon,
                text,
                receiver,
                member,
                shortcut,
            )
        else:
            action = old_add_action(self, *args)
    else:
        action = old_add_action(self, *args)

    return action


def static_method_kwargs_wrapper(func, from_kwarg_name, to_kwarg_name):
    """
    Helper function to manage `from_kwarg_name` to `to_kwarg_name` kwargs name changes in static methods.

    Makes static methods accept the `from_kwarg_name` kwarg as `to_kwarg_name`.
    """

    @staticmethod
    @wraps(func)
    def _from_kwarg_name_to_kwarg_name_(*args, **kwargs):
        if from_kwarg_name in kwargs:
            kwargs[to_kwarg_name] = kwargs.pop(from_kwarg_name)
        return func(*args, **kwargs)

    return _from_kwarg_name_to_kwarg_name_



# -----------------------------------------------------------------------------
# Copyright © 2014-2015 Colin Duquesnoy
# Copyright © 2009- The Spyder Development Team
#
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
# -----------------------------------------------------------------------------

"""Provides widget classes and functions."""
from functools import partialmethod

from packaging.version import parse

from . import PYQT5, PYQT6, PYSIDE2, PYSIDE6
from . import QT_VERSION as _qt_version
from ._utils import (
    add_action,
    getattr_missing_optional_dep,
    possibly_static_exec,
    static_method_kwargs_wrapper,
)

_missing_optional_names = {}


def __getattr__(name):
    """Custom getattr to chain and wrap errors due to missing optional deps."""
    raise getattr_missing_optional_dep(
        name,
        module_name=__name__,
        optional_names=_missing_optional_names,
    )


if PYQT5:
    from PyQt5.QtWidgets import *
elif PYQT6:
    from PyQt6 import QtWidgets
    from PyQt6.QtGui import (
        QActionGroup,
        QFileSystemModel,
        QShortcut,
        QUndoCommand,
    )
    from PyQt6.QtWidgets import *

    from qtpy.QtGui import QAction  # See spyder-ide/qtpy#461

    # Attempt to import QOpenGLWidget, but if that fails,
    # don't raise an exception until the name is explicitly accessed.
    # See https://github.com/spyder-ide/qtpy/pull/387/
    try:
        from PyQt6.QtOpenGLWidgets import QOpenGLWidget
    except ImportError as error:
        _missing_optional_names["QOpenGLWidget"] = {
            "name": "PyQt6.QtOpenGLWidgets",
            "missing_package": "pyopengl",
            "import_error": error,
        }

    # Map missing/renamed methods
    QTextEdit.setTabStopWidth = (
        lambda self, *args, **kwargs: self.setTabStopDistance(*args, **kwargs)
    )
    QTextEdit.tabStopWidth = (
        lambda self, *args, **kwargs: self.tabStopDistance(*args, **kwargs)
    )
    QTextEdit.print_ = lambda self, *args, **kwargs: self.print(
        *args,
        **kwargs,
    )
    QPlainTextEdit.setTabStopWidth = (
        lambda self, *args, **kwargs: self.setTabStopDistance(*args, **kwargs)
    )
    QPlainTextEdit.tabStopWidth = (
        lambda self, *args, **kwargs: self.tabStopDistance(*args, **kwargs)
    )
    QPlainTextEdit.print_ = lambda self, *args, **kwargs: self.print(
        *args,
        **kwargs,
    )
    QApplication.exec_ = lambda *args, **kwargs: possibly_static_exec(
        QApplication,
        *args,
        **kwargs,
    )
    QDialog.exec_ = lambda self, *args, **kwargs: self.exec(*args, **kwargs)
    QMenu.exec_ = lambda *args, **kwargs: possibly_static_exec(
        QMenu,
        *args,
        **kwargs,
    )
    QLineEdit.getTextMargins = lambda self: (
        self.textMargins().left(),
        self.textMargins().top(),
        self.textMargins().right(),
        self.textMargins().bottom(),
    )

    # Add removed definition for `QFileDialog.Options` as an alias of `QFileDialog.Option`
    # passing as default value 0 in the same way PySide6 6.5+ does.
    # Note that for PyQt5 and PySide2 those definitions are two different classes
    # (one is the flag definition and the other the enum definition)
    QFileDialog.Options = lambda value=0: QFileDialog.Option(value)

    # Allow unscoped access for enums inside the QtWidgets module
    from .enums_compat import promote_enums

    promote_enums(QtWidgets)
    del QtWidgets
elif PYSIDE2:
    from PySide2.QtWidgets import *
elif PYSIDE6:
    from PySide6.QtGui import QActionGroup, QShortcut, QUndoCommand
    from PySide6.QtWidgets import *

    from qtpy.QtGui import QAction  # See spyder-ide/qtpy#461

    # Attempt to import QOpenGLWidget, but if that fails,
    # don't raise an exception until the name is explicitly accessed.
    # See https://github.com/spyder-ide/qtpy/pull/387/
    try:
        from PySide6.QtOpenGLWidgets import QOpenGLWidget
    except ImportError as error:
        _missing_optional_names["QOpenGLWidget"] = {
            "name": "PySide6.QtOpenGLWidgets",
            "missing_package": "pyopengl",
            "import_error": error,
        }

    # Map missing/renamed methods
    QTextEdit.setTabStopWidth = (
        lambda self, *args, **kwargs: self.setTabStopDistance(*args, **kwargs)
    )
    QTextEdit.tabStopWidth = (
        lambda self, *args, **kwargs: self.tabStopDistance(*args, **kwargs)
    )
    QPlainTextEdit.setTabStopWidth = (
        lambda self, *args, **kwargs: self.setTabStopDistance(*args, **kwargs)
    )
    QPlainTextEdit.tabStopWidth = (
        lambda self, *args, **kwargs: self.tabStopDistance(*args, **kwargs)
    )
    QLineEdit.getTextMargins = lambda self: (
        self.textMargins().left(),
        self.textMargins().top(),
        self.textMargins().right(),
        self.textMargins().bottom(),
    )

    # Map DeprecationWarning methods
    QApplication.exec_ = lambda *args, **kwargs: possibly_static_exec(
        QApplication,
        *args,
        **kwargs,
    )
    QDialog.exec_ = lambda self, *args, **kwargs: self.exec(*args, **kwargs)
    QMenu.exec_ = lambda *args, **kwargs: possibly_static_exec(
        QMenu,
        *args,
        **kwargs,
    )

    # Passing as default value 0 in the same way PySide6 < 6.3.2 does for the `QFileDialog.Options` definition.
    if parse(_qt_version) > parse("6.3"):
        QFileDialog.Options = lambda value=0: QFileDialog.Option(value)


if PYSIDE2 or PYSIDE6:
    # Make PySide2/6 `QFileDialog` static methods accept the `directory` kwarg as `dir`
    QFileDialog.getExistingDirectory = static_method_kwargs_wrapper(
        QFileDialog.getExistingDirectory,
        "directory",
        "dir",
    )
    QFileDialog.getOpenFileName = static_method_kwargs_wrapper(
        QFileDialog.getOpenFileName,
        "directory",
        "dir",
    )
    QFileDialog.getOpenFileNames = static_method_kwargs_wrapper(
        QFileDialog.getOpenFileNames,
        "directory",
        "dir",
    )
    QFileDialog.getSaveFileName = static_method_kwargs_wrapper(
        QFileDialog.getSaveFileName,
        "directory",
        "dir",
    )
else:
    # Make PyQt5/6 `QFileDialog` static methods accept the `dir` kwarg as `directory`
    QFileDialog.getExistingDirectory = static_method_kwargs_wrapper(
        QFileDialog.getExistingDirectory,
        "dir",
        "directory",
    )
    QFileDialog.getOpenFileName = static_method_kwargs_wrapper(
        QFileDialog.getOpenFileName,
        "dir",
        "directory",
    )
    QFileDialog.getOpenFileNames = static_method_kwargs_wrapper(
        QFileDialog.getOpenFileNames,
        "dir",
        "directory",
    )
    QFileDialog.getSaveFileName = static_method_kwargs_wrapper(
        QFileDialog.getSaveFileName,
        "dir",
        "directory",
    )

# Make `addAction` compatible with Qt6 >= 6.4
if PYQT5 or PYSIDE2 or parse(_qt_version) < parse("6.4"):

    class _QMenu(QMenu):
        old_add_action = QMenu.addAction

        def addAction(self, *args):
            return add_action(
                self,
                *args,
                old_add_action=_QMenu.old_add_action,
            )

    _menu_add_action = partialmethod(
        add_action,
        old_add_action=QMenu.addAction,
    )
    QMenu.addAction = _menu_add_action
    # Despite the previous line!
    if QMenu.addAction is not _menu_add_action:
        QMenu = _QMenu

    class _QToolBar(QToolBar):
        old_add_action = QToolBar.addAction

        def addAction(self, *args):
            return add_action(
                self,
                *args,
                old_add_action=_QToolBar.old_add_action,
            )

    _toolbar_add_action = partialmethod(
        add_action,
        old_add_action=QToolBar.addAction,
    )
    QToolBar.addAction = _toolbar_add_action
    # Despite the previous line!
    if QToolBar.addAction is not _toolbar_add_action:
        QToolBar = _QToolBar



# -----------------------------------------------------------------------------
# Copyright © 2009- The Spyder Development Team
#
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
# -----------------------------------------------------------------------------

"""Provides Linux-specific utilities"""

import sys

from . import (
    PYQT5,
    PYQT6,
    PYSIDE2,
    PYSIDE6,
    QtModuleNotInOSError,
    QtModuleNotInQtVersionError,
)

if sys.platform == "linux":
    if PYQT5:
        from PyQt5.QtX11Extras import *
    elif PYQT6:
        raise QtModuleNotInQtVersionError(name="QtX11Extras")
    elif PYSIDE2:
        from PySide2.QtX11Extras import *
    elif PYSIDE6:
        raise QtModuleNotInQtVersionError(name="QtX11Extras")
else:
    raise QtModuleNotInOSError(name="QtX11Extras")



import pytest

from qtpy.tests.utils import pytest_importorskip


def test_qtbluetooth():
    """Test the qtpy.QtBluetooth namespace"""
    QtBluetooth = pytest_importorskip("qtpy.QtBluetooth")

    assert QtBluetooth.QBluetooth is not None
    assert QtBluetooth.QBluetoothDeviceInfo is not None
    assert QtBluetooth.QBluetoothServer is not None
    assert QtBluetooth.QBluetoothSocket is not None
    assert QtBluetooth.QBluetoothAddress is not None
    assert QtBluetooth.QBluetoothUuid is not None
    assert QtBluetooth.QBluetoothServiceDiscoveryAgent is not None



import contextlib
import os
import subprocess
import sys

import pytest

from qtpy import API_NAMES, QtCore, QtGui, QtWidgets
from qtpy.tests.utils import pytest_importorskip

with contextlib.suppress(Exception):
    # removed in qt 6.0
    from qtpy import QtWebEngineWidgets


def assert_pyside2():
    """
    Make sure that we are using PySide
    """
    import PySide2

    assert QtCore.QEvent is PySide2.QtCore.QEvent
    assert QtGui.QPainter is PySide2.QtGui.QPainter
    assert QtWidgets.QWidget is PySide2.QtWidgets.QWidget
    assert (
        QtWebEngineWidgets.QWebEnginePage
        is PySide2.QtWebEngineWidgets.QWebEnginePage
    )
    assert os.environ["QT_API"] == "pyside2"


def assert_pyside6():
    """
    Make sure that we are using PySide
    """
    import PySide6

    assert QtCore.QEvent is PySide6.QtCore.QEvent
    assert QtGui.QPainter is PySide6.QtGui.QPainter
    assert QtWidgets.QWidget is PySide6.QtWidgets.QWidget
    # Only valid for qt>=6.2
    # assert QtWebEngineWidgets.QWebEnginePage is PySide6.QtWebEngineCore.QWebEnginePage
    assert os.environ["QT_API"] == "pyside6"


def assert_pyqt5():
    """
    Make sure that we are using PyQt5
    """
    import PyQt5

    assert QtCore.QEvent is PyQt5.QtCore.QEvent
    assert QtGui.QPainter is PyQt5.QtGui.QPainter
    assert QtWidgets.QWidget is PyQt5.QtWidgets.QWidget
    assert os.environ["QT_API"] == "pyqt5"


def assert_pyqt6():
    """
    Make sure that we are using PyQt6
    """
    import PyQt6

    assert QtCore.QEvent is PyQt6.QtCore.QEvent
    assert QtGui.QPainter is PyQt6.QtGui.QPainter
    assert QtWidgets.QWidget is PyQt6.QtWidgets.QWidget
    assert os.environ["QT_API"] == "pyqt6"


def test_qt_api():
    """
    If QT_API is specified, we check that the correct Qt wrapper was used
    """

    QT_API = os.environ.get("QT_API", "").lower()

    if QT_API == "pyqt5":
        assert_pyqt5()
    elif QT_API == "pyside2":
        assert_pyside2()
    elif QT_API == "pyqt6":
        assert_pyqt6()
    elif QT_API == "pyside6":
        assert_pyside6()
    else:
        # If the tests are run locally, USE_QT_API and QT_API may not be
        # defined, but we still want to make sure qtpy is behaving sensibly.
        # We should then be loading, in order of decreasing preference, PyQt5,
        # PySide2, PyQt6 and PySide6.
        try:
            import PyQt5
        except ImportError:
            try:
                import PySide2
            except ImportError:
                try:
                    import PyQt6
                except ImportError:
                    import PySide6

                    assert_pyside6()
                else:
                    assert_pyqt6()
            else:
                assert_pyside2()
        else:
            assert_pyqt5()


@pytest.mark.parametrize("api", API_NAMES.values())
def test_qt_api_environ(api):
    """
    If no QT_API is specified but some Qt is imported, ensure QT_API is set properly.
    """
    mod = f"{api}.QtCore"
    pytest_importorskip(mod, reason=f"Requires {api}")
    # clean env
    env = os.environ.copy()
    for key in ("QT_API", "USE_QT_API"):
        if key in env:
            del env[key]
    cmd = f"""
import {mod}
from qtpy import API
import os
print(API)
print(os.environ['QT_API'])
"""
    output = subprocess.check_output([sys.executable, "-c", cmd], env=env)
    got_api, env_qt_api = output.strip().decode("utf-8").splitlines()
    assert got_api == api.lower()
    assert env_qt_api == api.lower()
    # Also ensure we raise a nice error
    env["QT_API"] = "bad"
    cmd = """
try:
    import qtpy
except ValueError as exc:
    assert 'Specified QT_API' in str(exc), str(exc)
else:
    raise AssertionError('QtPy imported despite bad QT_API')
"""
    subprocess.check_call([sys.executable, "-Oc", cmd], env=env)



import contextlib
import os
import sys
import warnings

import pytest
from packaging.version import parse

from qtpy import PYSIDE2, PYSIDE6, PYSIDE_VERSION, QtWidgets
from qtpy.QtWidgets import QComboBox
from qtpy.tests.utils import pytest_importorskip, using_conda

if PYSIDE2:
    pytest_importorskip("pyside2uic", reason="pyside2uic not installed")

from qtpy import uic

QCOMBOBOX_SUBCLASS = """
from qtpy.QtWidgets import QComboBox
class _QComboBoxSubclass(QComboBox):
    pass
"""


@contextlib.contextmanager
def enabled_qcombobox_subclass(temp_dir_path):
    """
    Context manager that sets up a temporary module with a QComboBox subclass
    and then removes it once we are done.
    """

    with open(
        temp_dir_path / "qcombobox_subclass.py",
        mode="w",
        encoding="utf-8",
    ) as f:
        f.write(QCOMBOBOX_SUBCLASS)

    sys.path.insert(0, str(temp_dir_path))

    yield

    sys.path.pop(0)


def test_load_ui(qtbot):
    """
    Make sure that the patched loadUi function behaves as expected with a
    simple .ui file.
    """
    with warnings.catch_warnings():
        warnings.filterwarnings(
            "ignore",
            category=DeprecationWarning,
            message=".*mode.*",
        )
        ui = uic.loadUi(os.path.join(os.path.dirname(__file__), "test.ui"))

    assert isinstance(ui.pushButton, QtWidgets.QPushButton)
    assert isinstance(ui.comboBox, QComboBox)


@pytest.mark.skipif(
    PYSIDE6
    and using_conda()
    and parse(PYSIDE_VERSION) < parse("6.5")
    and (sys.platform in ("darwin", "linux")),
    reason="pyside6-uic command not contained in all conda-forge packages.",
)
def test_load_ui_type(qtbot):
    """
    Make sure that the patched loadUiType function behaves as expected with a
    simple .ui file.
    """
    with warnings.catch_warnings():
        warnings.filterwarnings(
            "ignore",
            category=DeprecationWarning,
            message=".*mode.*",
        )
        ui_type, ui_base_type = uic.loadUiType(
            os.path.join(os.path.dirname(__file__), "test.ui"),
        )
    assert ui_type.__name__ == "Ui_Form"

    class Widget(ui_base_type, ui_type):
        def __init__(self):
            super().__init__()
            self.setupUi(self)

    ui = Widget()
    assert isinstance(ui, QtWidgets.QWidget)
    assert isinstance(ui.pushButton, QtWidgets.QPushButton)
    assert isinstance(ui.comboBox, QComboBox)


def test_load_ui_custom_auto(qtbot, tmp_path):
    """
    Test that we can load a .ui file with custom widgets without having to
    explicitly specify a dictionary of custom widgets, even in the case of
    PySide.
    """
    with enabled_qcombobox_subclass(tmp_path):
        from qcombobox_subclass import _QComboBoxSubclass

        with warnings.catch_warnings():
            warnings.filterwarnings(
                "ignore",
                category=DeprecationWarning,
                message=".*mode.*",
            )
            ui = uic.loadUi(
                os.path.join(os.path.dirname(__file__), "test_custom.ui"),
            )

    assert isinstance(ui.pushButton, QtWidgets.QPushButton)
    assert isinstance(ui.comboBox, _QComboBoxSubclass)


def test_load_full_uic():
    """Test that we load the full uic objects."""
    QT_API = os.environ.get("QT_API", "").lower()
    if QT_API.startswith("pyside"):
        assert hasattr(uic, "loadUi")
        assert hasattr(uic, "loadUiType")
    else:
        objects = [
            "compileUi",
            "compileUiDir",
            "loadUi",
            "loadUiType",
            "widgetPluginPath",
        ]
        assert all(hasattr(uic, o) for o in objects)



"""Test QtMultimediaWidgets."""


from qtpy import PYQT5, PYSIDE2


def test_qtmultimediawidgets():
    """Test the qtpy.QtMultimediaWidgets namespace"""
    from qtpy import QtMultimediaWidgets

    if PYQT5 or PYSIDE2:
        assert QtMultimediaWidgets.QCameraViewfinder is not None
        # assert QtMultimediaWidgets.QVideoWidgetControl is not None
    assert QtMultimediaWidgets.QGraphicsVideoItem is not None
    assert QtMultimediaWidgets.QVideoWidget is not None



import pytest

from qtpy import PYSIDE2
from qtpy.tests.utils import pytest_importorskip


@pytest.mark.skipif(PYSIDE2, reason="QtDesigner is not available in PySide2")
def test_qtdesigner():
    """Test the qtpy.QtDesigner namespace."""
    QtDesigner = pytest_importorskip("qtpy.QtDesigner")

    assert QtDesigner.QAbstractExtensionFactory is not None
    assert QtDesigner.QAbstractExtensionManager is not None
    assert QtDesigner.QDesignerActionEditorInterface is not None
    assert QtDesigner.QDesignerContainerExtension is not None
    assert QtDesigner.QDesignerCustomWidgetCollectionInterface is not None
    assert QtDesigner.QDesignerCustomWidgetInterface is not None
    assert QtDesigner.QDesignerFormEditorInterface is not None
    assert QtDesigner.QDesignerFormWindowCursorInterface is not None
    assert QtDesigner.QDesignerFormWindowInterface is not None
    assert QtDesigner.QDesignerFormWindowManagerInterface is not None
    assert QtDesigner.QDesignerMemberSheetExtension is not None
    assert QtDesigner.QDesignerObjectInspectorInterface is not None
    assert QtDesigner.QDesignerPropertyEditorInterface is not None
    assert QtDesigner.QDesignerPropertySheetExtension is not None
    assert QtDesigner.QDesignerTaskMenuExtension is not None
    assert QtDesigner.QDesignerWidgetBoxInterface is not None
    assert QtDesigner.QExtensionFactory is not None
    assert QtDesigner.QExtensionManager is not None
    assert QtDesigner.QFormBuilder is not None



# -----------------------------------------------------------------------------
# Copyright © 2014-2015 Colin Duquesnoy
# Copyright © 2009- The Spyder development Team
#
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
# -----------------------------------------------------------------------------

"""Provides QtWebEngine classes and functions."""

from . import (
    PYQT5,
    PYQT6,
    PYSIDE2,
    PYSIDE6,
    QtModuleNotInQtVersionError,
    QtModuleNotInstalledError,
)

if PYQT5:
    try:
        from PyQt5.QtWebEngine import *
    except ModuleNotFoundError as error:
        raise QtModuleNotInstalledError(
            name="QtWebEngine",
            missing_package="PyQtWebEngine",
        ) from error
elif PYQT6:
    raise QtModuleNotInQtVersionError(name="QtWebEngine")
elif PYSIDE2:
    from PySide2.QtWebEngine import *
elif PYSIDE6:
    raise QtModuleNotInQtVersionError(name="QtWebEngine")



# -----------------------------------------------------------------------------
# Copyright © 2014-2015 Colin Duquesnoy
# Copyright © 2009- The Spyder Development Team
#
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
# -----------------------------------------------------------------------------

"""Provides QtTest and functions"""

from . import PYQT5, PYQT6, PYSIDE2, PYSIDE6

if PYQT5:
    from PyQt5.QtTest import *
elif PYQT6:
    from PyQt6 import QtTest
    from PyQt6.QtTest import *

    # Allow unscoped access for enums inside the QtTest module
    from .enums_compat import promote_enums

    promote_enums(QtTest)
    del QtTest
elif PYSIDE2:
    from PySide2.QtTest import *
elif PYSIDE6:
    from PySide6.QtTest import *



"""Package used for testing the deferred import error mechanism."""


# See https://github.com/spyder-ide/qtpy/pull/387/


from qtpy._utils import getattr_missing_optional_dep

from .optional_dep import ExampleClass

_missing_optional_names = {}


try:
    from .optional_dep import MissingClass
except ImportError as error:
    _missing_optional_names["MissingClass"] = {
        "name": "optional_dep.MissingClass",
        "missing_package": "test_package_please_ignore",
        "import_error": error,
    }


def __getattr__(name):
    """Custom getattr to chain and wrap errors due to missing optional deps."""
    raise getattr_missing_optional_dep(
        name,
        module_name=__name__,
        optional_names=_missing_optional_names,
    )



# -----------------------------------------------------------------------------
# Copyright © 2014-2015 Colin Duquesnoy
# Copyright © 2009- The Spyder Development Team
#
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
# -----------------------------------------------------------------------------

"""Provides QtCore classes and functions."""
import contextlib
from typing import TYPE_CHECKING

from packaging.version import parse

from . import PYQT5, PYQT6, PYSIDE2, PYSIDE6
from . import QT_VERSION as _qt_version
from ._utils import possibly_static_exec, possibly_static_exec_

if PYQT5:
    from PyQt5.QtCore import *
    from PyQt5.QtCore import pyqtBoundSignal as SignalInstance
    from PyQt5.QtCore import pyqtProperty as Property
    from PyQt5.QtCore import pyqtSignal as Signal
    from PyQt5.QtCore import pyqtSlot as Slot

    try:
        from PyQt5.QtCore import Q_ENUM as QEnum

        del Q_ENUM
    except ImportError:  # fallback for Qt5.9
        from PyQt5.QtCore import Q_ENUMS as QEnum

        del Q_ENUMS
    from PyQt5.QtCore import QT_VERSION_STR as __version__

    # Those are imported from `import *`
    del pyqtSignal, pyqtBoundSignal, pyqtSlot, pyqtProperty, QT_VERSION_STR

elif PYQT6:
    from PyQt6 import QtCore
    from PyQt6.QtCore import *
    from PyQt6.QtCore import QT_VERSION_STR as __version__
    from PyQt6.QtCore import pyqtBoundSignal as SignalInstance
    from PyQt6.QtCore import pyqtEnum as QEnum
    from PyQt6.QtCore import pyqtProperty as Property
    from PyQt6.QtCore import pyqtSignal as Signal
    from PyQt6.QtCore import pyqtSlot as Slot

    # For issue #311
    # Seems like there is an error with sip. Without first
    # trying to import `PyQt6.QtGui.Qt`, some functions like
    # `PyQt6.QtCore.Qt.mightBeRichText` are missing.
    if not TYPE_CHECKING:
        with contextlib.suppress(ImportError):
            from PyQt6.QtGui import Qt

    # Map missing methods
    QCoreApplication.exec_ = lambda *args, **kwargs: possibly_static_exec(
        QCoreApplication,
        *args,
        **kwargs,
    )
    QEventLoop.exec_ = lambda self, *args, **kwargs: self.exec(*args, **kwargs)
    QThread.exec_ = lambda self, *args, **kwargs: self.exec(*args, **kwargs)

    # Those are imported from `import *`
    del (
        pyqtSignal,
        pyqtBoundSignal,
        pyqtSlot,
        pyqtProperty,
        pyqtEnum,
        QT_VERSION_STR,
    )

    # Allow unscoped access for enums inside the QtCore module
    from .enums_compat import promote_enums

    promote_enums(QtCore)
    del QtCore

    # Alias deprecated ItemDataRole enum values removed in Qt6
    Qt.BackgroundColorRole = (
        Qt.ItemDataRole.BackgroundColorRole
    ) = Qt.BackgroundRole
    Qt.TextColorRole = Qt.ItemDataRole.TextColorRole = Qt.ForegroundRole

    # Alias for MiddleButton removed in PyQt6 but available in PyQt5, PySide2 and PySide6
    Qt.MidButton = Qt.MiddleButton
    Qt.MouseButton.MidButton = Qt.MiddleButton

    # Add removed definition for `Qt.ItemFlags` as an alias of `Qt.ItemFlag`
    # passing as default value 0 in the same way PySide6 6.5+ does.
    # Note that for PyQt5 and PySide2 those definitions are two different classes
    # (one is the flag definition and the other the enum definition)
    Qt.ItemFlags = lambda value=0: Qt.ItemFlag(value)

elif PYSIDE2:
    import PySide2.QtCore
    from PySide2.QtCore import *

    __version__ = PySide2.QtCore.__version__

    # Missing QtGui utility functions on Qt
    if getattr(Qt, "mightBeRichText", None) is None:
        try:
            from PySide2.QtGui import Qt as guiQt

            Qt.mightBeRichText = guiQt.mightBeRichText
            del guiQt
        except ImportError:
            # Fails with PySide2 5.12.0
            pass

    QCoreApplication.exec = lambda *args, **kwargs: possibly_static_exec_(
        QCoreApplication,
        *args,
        **kwargs,
    )
    QEventLoop.exec = lambda self, *args, **kwargs: self.exec_(*args, **kwargs)
    QThread.exec = lambda self, *args, **kwargs: self.exec_(*args, **kwargs)
    QTextStreamManipulator.exec = lambda self, *args, **kwargs: self.exec_(
        *args,
        **kwargs,
    )

elif PYSIDE6:
    import PySide6.QtCore
    from PySide6.QtCore import *

    __version__ = PySide6.QtCore.__version__

    # Missing QtGui utility functions on Qt
    if getattr(Qt, "mightBeRichText", None) is None:
        from PySide6.QtGui import Qt as guiQt

        Qt.mightBeRichText = guiQt.mightBeRichText
        del guiQt

    # Alias deprecated ItemDataRole enum values removed in Qt6
    Qt.BackgroundColorRole = (
        Qt.ItemDataRole.BackgroundColorRole
    ) = Qt.BackgroundRole
    Qt.TextColorRole = Qt.ItemDataRole.TextColorRole = Qt.ForegroundRole
    Qt.MidButton = Qt.MiddleButton
    Qt.MouseButton.MidButton = Qt.MiddleButton

    # Map DeprecationWarning methods
    QCoreApplication.exec_ = lambda *args, **kwargs: possibly_static_exec(
        QCoreApplication,
        *args,
        **kwargs,
    )
    QEventLoop.exec_ = lambda self, *args, **kwargs: self.exec(*args, **kwargs)
    QThread.exec_ = lambda self, *args, **kwargs: self.exec(*args, **kwargs)
    QTextStreamManipulator.exec_ = lambda self, *args, **kwargs: self.exec(
        *args,
        **kwargs,
    )

    # Passing as default value 0 in the same way PySide6 6.3.2 does for the `Qt.ItemFlags` definition.
    if parse(_qt_version) > parse("6.3"):
        Qt.ItemFlags = lambda value=0: Qt.ItemFlag(value)

# For issue #153 and updated for issue #305
if PYQT5 or PYQT6:
    QDate.toPython = lambda self, *args, **kwargs: self.toPyDate(
        *args,
        **kwargs,
    )
    QDateTime.toPython = lambda self, *args, **kwargs: self.toPyDateTime(
        *args,
        **kwargs,
    )
    QTime.toPython = lambda self, *args, **kwargs: self.toPyTime(
        *args,
        **kwargs,
    )
if PYSIDE2 or PYSIDE6:
    QDate.toPyDate = lambda self, *args, **kwargs: self.toPython(
        *args,
        **kwargs,
    )
    QDateTime.toPyDateTime = lambda self, *args, **kwargs: self.toPython(
        *args,
        **kwargs,
    )
    QTime.toPyTime = lambda self, *args, **kwargs: self.toPython(
        *args,
        **kwargs,
    )

# Mirror https://github.com/spyder-ide/qtpy/pull/393
if PYQT5 or PYSIDE2:
    QLibraryInfo.path = QLibraryInfo.location
    QLibraryInfo.LibraryPath = QLibraryInfo.LibraryLocation
if PYQT6 or PYSIDE6:
    QLibraryInfo.location = QLibraryInfo.path
    QLibraryInfo.LibraryLocation = QLibraryInfo.LibraryPath
